/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/base-prefs.h>
#include <pan/base/debug.h>
#include <pan/base/log.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/util-file.h>

#include <pan/globals.h>
#include <pan/gui.h>
#include <pan/queue.h>
#include <pan/pan-file-entry.h>
#include <pan/task-bodies.h>
#include <pan/task-save.h>
#include <pan/save-ui.h>
#include <pan/text.h>
#include <pan/util.h>

/*********************
**********************  Private Function Prototypes
*********************/

static gchar * last_save_as_path = NULL;

static gboolean append_groupname = FALSE;

static gboolean back_of_queue = TRUE;

static gboolean save_attachments = TRUE;
static gboolean use_subject = FALSE;
static gboolean use_encoded = TRUE;
static gboolean use_custom = FALSE;

static gboolean save_bodies = FALSE;
static gboolean use_message_id = FALSE;
static gboolean use_subject_for_body = FALSE;
static gboolean use_custom_for_body = FALSE;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

typedef struct
{
	GtkWidget * default_dir_rb;
	GtkWidget * group_default_dir_rb;
	GtkWidget * user_specified_rb;
	GtkWidget * user_specified_entry;
	GtkWidget * append_groupname_tb;

	GtkWidget * body_filename_off_rb;
	GtkWidget * body_filename_message_id_rb;
	GtkWidget * body_filename_subject_rb;
	GtkWidget * body_filename_custom_rb;
	GtkWidget * body_filename_custom_entry;

	GtkWidget * decode_filename_encoded_rb;
	GtkWidget * decode_filename_subject_rb;
	GtkWidget * decode_filename_custom_rb;
	GtkWidget * decode_filename_custom_entry;
	GtkWidget * decode_filename_off_rb;

	GtkWidget * front_of_queue_rb;
	GtkWidget * back_of_queue_rb;

	Group  *group;

	GSList *gslists_of_unowned_articles;
}
SaveAsDialogStruct;

#define IS_ACTIVE(w) gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))

char*
expand_download_dir (const char * dir, const char * groupname)
{
	char * retval = g_strdup (dir);

	if (is_nonempty_string(groupname))
	{
		if (strstr (dir, "%g") != NULL)
		{
			replace_gstr (&retval, pan_substitute (retval, "%g", groupname));
		}

		if (strstr (dir, "%G") != NULL)
		{
			char tmp[PATH_MAX], *pch;
			g_snprintf (tmp, sizeof(tmp), "%s.", groupname);
			for (pch=tmp; *pch; ++pch)
				if (*pch == '.')
					*pch = G_DIR_SEPARATOR;
			replace_gstr (&retval, pan_substitute (retval, "%G", tmp));
		}
	}

	return retval;
}

static void
save_attachment_as_dialog_response_cb (GtkDialog   * dialog,
                                       int           response,
                                       gpointer      user_data)
{
	SaveAsDialogStruct* save = (SaveAsDialogStruct*)user_data;
	GSList * l;
	GSList * tasks = NULL;
	char * path;
	const char * custom_body_fname = NULL;
	const char * custom_attachment_fname = NULL;
	debug_enter ("save_attachment_as_dialog_clicked_cb");

	if (response == GTK_RESPONSE_CANCEL)
	{
		pan_lock();
		gtk_widget_destroy (GTK_WIDGET(dialog));
		pan_unlock();
		return;
	}

	/**
	***  Get Path
	**/

	path = NULL;
	if (IS_ACTIVE(save->user_specified_rb)) {
		const char * cpath = pan_file_entry_get (save->user_specified_entry);
		path = g_strdup (cpath);
		g_strstrip (path);
		if (path!=NULL && *path=='~')
			replace_gstr (&path, g_strconcat (g_get_home_dir(), path+1, NULL));
	}
	else if ((save->group_default_dir_rb!=NULL) && IS_ACTIVE(save->group_default_dir_rb)) {
		path = g_strdup (save->group->download_dir);
	}
	else {
		path = g_strdup (download_dir);
	}

	if (IS_ACTIVE(save->append_groupname_tb))
		replace_gstr (&path, g_strdup_printf ("%s%c%s", path, G_DIR_SEPARATOR, save->group->name));

	/**
	***  Get Body Filename
	**/

	save_bodies = !IS_ACTIVE(save->body_filename_off_rb);
	use_message_id = IS_ACTIVE(save->body_filename_message_id_rb);
	use_subject_for_body = IS_ACTIVE(save->body_filename_subject_rb);
	use_custom_for_body = IS_ACTIVE(save->body_filename_custom_rb);
	custom_body_fname = pan_file_entry_get (save->body_filename_custom_entry);

	/**
	***  Get Attachment Filenames
	**/

	save_attachments = !IS_ACTIVE(save->decode_filename_off_rb);
	use_encoded = IS_ACTIVE(save->decode_filename_encoded_rb);
	use_subject = IS_ACTIVE(save->decode_filename_subject_rb);
	use_custom = IS_ACTIVE(save->decode_filename_custom_rb);
	custom_attachment_fname = pan_file_entry_get (save->decode_filename_custom_entry);

	/**
	***  Queue 'em
	**/

	for (l=save->gslists_of_unowned_articles; l!=NULL; l=l->next)
	{
		GSList * m;
		GSList * articles = (GSList*) l->data;
		const Article * article = ARTICLE(articles->data);
		const gchar * subject = article_get_subject (article);
		char * expanded_path = NULL;
		GPtrArray * tmp;
		PanObject * task;

		/* make an array of articles */
		tmp = g_ptr_array_sized_new (64);
		for (m=(GSList*)l->data; m!=NULL; m=m->next)
			g_ptr_array_add (tmp, m->data);
		task_save_weed_duplicates (tmp);
		task = task_save_new_from_articles (subject, (const Article**)tmp->pdata, tmp->len);
		g_ptr_array_free (tmp, TRUE);

		expanded_path = expand_download_dir (path, article->group->name);

		if (save_attachments)
		{
			const char * filename = NULL;
			if (use_subject)
				filename = subject;
			else if (use_custom)
				filename = custom_attachment_fname;

			task_save_set_attachments (TASK_SAVE(task), expanded_path, filename);
		}

		if (save_bodies)
		{
			const char * filename = NULL;
			if (use_custom_for_body)
				filename = custom_body_fname;
			else if (use_message_id)
				filename = article_get_message_id (article);
			else if (use_subject_for_body)
				filename = subject;
			task_save_set_bodies (TASK_SAVE(task), expanded_path, filename);
		}

		tasks = g_slist_prepend (tasks, task);
		g_free (expanded_path);
	}

	if (tasks != NULL)
	{
		tasks = g_slist_reverse (tasks);
		queue_insert_tasks (tasks, back_of_queue ? -1 : 0);
	}

	/* cleanup */
	g_free (path);
	pan_lock();
	gtk_widget_destroy (GTK_WIDGET(dialog));
	pan_unlock();
	debug_exit ("save_attachment_as_dialog_clicked_cb");
}

static void
priority_toggled_cb (GtkToggleButton    * togglebutton,
                     gpointer              user_data)
{
	SaveAsDialogStruct *save = (SaveAsDialogStruct*) user_data;
	gboolean b = gtk_toggle_button_get_active (togglebutton);
	back_of_queue = togglebutton==GTK_TOGGLE_BUTTON(save->back_of_queue_rb) ? b : !b;
}

static void
append_groupname_toggled_cb (GtkToggleButton   *  togglebutton,
                             gpointer             user_data)
{
	append_groupname = gtk_toggle_button_get_active(togglebutton);
}


static void
save_attachment_as_dialog_destroy_cb (GtkDialog * d, gpointer user_data)
{
	GSList * l;
	SaveAsDialogStruct *save = (SaveAsDialogStruct*) user_data;

	/* free the save-as-dialog utility structure */
	for (l=save->gslists_of_unowned_articles; l!=NULL; l=l->next)
		g_slist_free ((GSList*)l->data);

	/* cleanup */
	g_slist_free (save->gslists_of_unowned_articles);
	g_free (save);
}

static void
user_specified_path_changed_cb (GtkEditable   * editable,
                                gpointer        user_data)
{
	replace_gstr (&last_save_as_path,
	              gtk_editable_get_chars (GTK_EDITABLE(editable), 0, -1));
}

static gint
entry_tickled_cb (GtkWidget * w, gpointer * event, gpointer user_data)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(user_data), TRUE);
	return FALSE;
}

static void
create_save_attachment_as_dialog (Group * group,
                                  GSList * gslists_of_unowned_articles)
{
	gchar * pch;
	GtkWidget * w;
	GtkWidget * hbox;
	GtkWidget * vbox;
	GtkWidget * frame;
	GtkWidget * entry;
	GtkWidget * dialog;
	SaveAsDialogStruct * data;

	data = g_new0 (SaveAsDialogStruct, 1);
	data->group = group;
	data->gslists_of_unowned_articles = gslists_of_unowned_articles;

	pan_lock ();

	/* create & parent the dialog */
	dialog = gtk_dialog_new_with_buttons (_("Pan: Save Articles As"),
	                                      GTK_WINDOW(Pan.window),
	                                      GTK_DIALOG_DESTROY_WITH_PARENT,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_OK, GTK_RESPONSE_OK,
	                                      NULL);
	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);

	/**
	***  "Location" Pane
	**/

	frame = gtk_frame_new (_("Location"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (FALSE, GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_SMALL);

	/* global default directory */
	w = gtk_radio_button_new_with_label (NULL, download_dir);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
	data->default_dir_rb = w;

	/* group default directory */
	if (is_nonempty_string (group->download_dir)) {
		w = gtk_radio_button_new_with_label_from_widget (
			GTK_RADIO_BUTTON(w), group->download_dir);
		gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
		data->group_default_dir_rb = w;
	}

	/* user-specified directory */
	hbox = gtk_hbox_new (FALSE, 0);
	w = gtk_radio_button_new_from_widget (GTK_RADIO_BUTTON(w));
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	data->user_specified_rb = w;

	entry = pan_file_entry_new (_("Save in Directory"));
	if (!is_nonempty_string(last_save_as_path))
		replace_gstr (&last_save_as_path, g_strdup(download_dir));
	pan_file_entry_set (entry, last_save_as_path);
	g_signal_connect (pan_file_entry_gtk_entry(entry), "focus_in_event",
	                  G_CALLBACK(entry_tickled_cb), data->user_specified_rb); 
	gtk_widget_set_usize (GTK_WIDGET(entry), 400, 0);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w),
				      last_save_as_path!=NULL);
	gtk_box_pack_start (GTK_BOX(hbox), entry, TRUE, TRUE, 0);
	data->user_specified_entry = entry;
	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


	/* append group name to the path? */
	pch = g_strdup_printf (_("Add \"/%s\" to path"), group->name);
	w = gtk_check_button_new_with_label (pch);
	g_free (pch);
	data->append_groupname_tb = w;
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), append_groupname);
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, GUI_PAD_SMALL);

	/* add "Location" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, TRUE, TRUE, GUI_PAD_SMALL);



	/***
	****  Message Pane
	***/


	frame = gtk_frame_new (_("Article Text"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (TRUE, GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_SMALL);

	/* don't save bodies */
	w = gtk_radio_button_new_with_label (NULL, _("Don't save article text"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), !save_bodies);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->body_filename_off_rb = w;

	/* use message-id */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use Message-ID header"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_message_id);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->body_filename_message_id_rb = w;

	/* use message-id */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use subject header"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_subject_for_body);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->body_filename_subject_rb = w;

	/* use custom */
	hbox = gtk_hbox_new (FALSE, 0);
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use custom filename"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_custom_for_body);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	data->body_filename_custom_rb = w;
	w = pan_file_entry_new (_("Specify filename"));
	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
	data->body_filename_custom_entry = w;
	g_signal_connect (pan_file_entry_gtk_entry(w), "focus_in_event",
	                  G_CALLBACK(entry_tickled_cb), data->body_filename_custom_rb); 
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	/* add "Bodies" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, TRUE, TRUE, GUI_PAD_SMALL);

	/***
	****  Attachments Pane
	***/

	frame = gtk_frame_new (_("Attachments"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (TRUE, GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_SMALL);

	/* don't save attachments */
	w = gtk_radio_button_new_with_label (NULL, _("Don't save attachments"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), !save_attachments);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->decode_filename_off_rb = w;

	/* use encoded filename */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use encoded filename"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_encoded);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->decode_filename_encoded_rb = w;

	/* use subject */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use article subject"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_subject);
	gtk_box_pack_start (GTK_BOX(vbox), w, TRUE, TRUE, 0);
	data->decode_filename_subject_rb = w;

	/* user-specified filenames */
	hbox = gtk_hbox_new (FALSE, 0);
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Use custom filename"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), use_custom);
	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
	data->decode_filename_custom_rb = w;
	w = pan_file_entry_new (_("Specify filename"));
	g_signal_connect (pan_file_entry_gtk_entry(w), "focus_in_event",
	                  G_CALLBACK(entry_tickled_cb), data->decode_filename_custom_rb); 
	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
	data->decode_filename_custom_entry = w;
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	/* add "Filename" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), frame, TRUE, TRUE, GUI_PAD_SMALL);

	/***
	****  Priority Pane
	***/

	/* "Priority" pane */
	frame = gtk_frame_new (_("Priority"));
	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
	vbox = gtk_vbox_new (TRUE, GUI_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER(vbox), GUI_PAD_SMALL);

	/* front of queue */
	w = gtk_radio_button_new_with_label (NULL, _("Add task to the front of the Task Manager"));
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), !back_of_queue);
	data->front_of_queue_rb = w;

	/* back of queue (default) */
	w = gtk_radio_button_new_with_label_from_widget (
		GTK_RADIO_BUTTON(w), _("Add task to the back of the Task Manager"));
	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), back_of_queue);
	data->back_of_queue_rb = w;

	/* add "Priority" to the dialog */
	gtk_container_add (GTK_CONTAINER(frame), vbox);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, TRUE, TRUE, GUI_PAD_SMALL);

	/* prep the dialog */
	g_signal_connect (GTK_OBJECT(dialog), "response",
	                  G_CALLBACK(save_attachment_as_dialog_response_cb), data);
	g_signal_connect (GTK_OBJECT(dialog), "destroy",
	                  G_CALLBACK(save_attachment_as_dialog_destroy_cb), data); 
	g_signal_connect (GTK_OBJECT(data->append_groupname_tb), "toggled",
	                  G_CALLBACK(append_groupname_toggled_cb), data);
	g_signal_connect (GTK_OBJECT(data->front_of_queue_rb), "toggled",
	                  G_CALLBACK(priority_toggled_cb), data);
	g_signal_connect (GTK_OBJECT(data->back_of_queue_rb), "toggled",
	                  G_CALLBACK(priority_toggled_cb), data);
	g_signal_connect (pan_file_entry_gtk_entry(entry), "changed",
	                  G_CALLBACK(user_specified_path_changed_cb), NULL);

	gtk_widget_show_all (dialog);
	pan_unlock ();
}



void
save_attachment_as (Group* group,
		    GSList* gslists_of_unowned_articles)
{
	create_save_attachment_as_dialog (group,
					  gslists_of_unowned_articles);
}
